package dev.shadowsoffire.apotheosis.adventure.compat;

import net.minecraftforge.registries.ForgeRegistries;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemInstance;
import mezz.jei.api.ingredients.subtypes.UidContext;
import dev.shadowsoffire.apotheosis.adventure.socket.SocketHelper;
import net.minecraft.client.gui.Font;
import net.minecraft.network.chat.FormattedText;
import net.minecraft.client.gui.GuiGraphics;
import mezz.jei.api.gui.ingredient.IRecipeSlotsView;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.IFocusGroup;
import mezz.jei.api.gui.builder.IRecipeLayoutBuilder;
import java.util.function.Function;
import java.util.Arrays;
import net.minecraft.world.item.Items;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import mezz.jei.api.ingredients.subtypes.IIngredientSubtypeInterpreter;
import net.minecraft.world.item.Item;
import mezz.jei.api.registration.ISubtypeRegistration;
import net.minecraft.world.level.block.Blocks;
import mezz.jei.api.registration.IRecipeCatalystRegistration;
import mezz.jei.api.recipe.category.IRecipeCategory;
import mezz.jei.api.registration.IRecipeCategoryRegistration;
import java.util.Iterator;
import java.util.stream.Stream;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Comparator;
import java.util.function.ToIntFunction;
import java.util.Collection;
import java.util.ArrayList;
import dev.shadowsoffire.apotheosis.Apoth;
import java.util.function.Predicate;
import java.util.Objects;
import dev.shadowsoffire.apotheosis.adventure.socket.ReactiveSmithingRecipe;
import net.minecraft.client.Minecraft;
import dev.shadowsoffire.apotheosis.adventure.socket.AddSocketsRecipe;
import mezz.jei.api.ingredients.IIngredientType;
import net.minecraft.network.chat.Component;
import mezz.jei.api.constants.VanillaTypes;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import net.minecraft.util.RandomSource;
import net.minecraft.world.level.levelgen.LegacyRandomSource;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.Gem;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import net.minecraft.world.level.ItemLike;
import dev.shadowsoffire.apotheosis.Apotheosis;
import mezz.jei.api.registration.IRecipeRegistration;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.ItemStack;
import java.util.List;
import dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvagingRecipe;
import net.minecraft.world.item.crafting.SmithingRecipe;
import mezz.jei.api.recipe.RecipeType;
import mezz.jei.api.JeiPlugin;
import mezz.jei.api.IModPlugin;

@JeiPlugin
public class AdventureJEIPlugin implements IModPlugin
{
    public static final RecipeType<SmithingRecipe> APO_SMITHING;
    public static final RecipeType<SalvagingRecipe> SALVAGING;
    public static final RecipeType<GemCuttingCategory.GemCuttingRecipe> GEM_CUTTING;
    private static final List<ItemStack> DUMMY_INPUTS;
    
    public ResourceLocation getPluginUid() {
        return new ResourceLocation("apotheosis", "adventure_module");
    }
    
    public void registerRecipes(final IRecipeRegistration reg) {
        if (!Apotheosis.enableAdventure) {
            return;
        }
        final ItemStack gem = new ItemStack((ItemLike)Adventure.Items.GEM.get());
        final Gem gemObj = (Gem)GemRegistry.INSTANCE.getRandomItem((RandomSource)new LegacyRandomSource(1854L));
        GemItem.setGem(gem, gemObj);
        AffixHelper.setRarity(gem, gemObj.getMaxRarity());
        reg.addIngredientInfo((Object)gem, (IIngredientType)VanillaTypes.ITEM_STACK, new Component[] { (Component)Component.m_237115_("info.apotheosis.socketing") });
        reg.addIngredientInfo((Object)new ItemStack((ItemLike)Adventure.Items.GEM_DUST.get()), (IIngredientType)VanillaTypes.ITEM_STACK, new Component[] { (Component)Component.m_237115_("info.apotheosis.gem_crushing") });
        reg.addIngredientInfo((Object)new ItemStack((ItemLike)Adventure.Items.SIGIL_OF_UNNAMING.get()), (IIngredientType)VanillaTypes.ITEM_STACK, new Component[] { (Component)Component.m_237115_("info.apotheosis.unnaming") });
        ApothSmithingCategory.registerExtension((Class<SmithingRecipe>)AddSocketsRecipe.class, (ApothSmithingCategory.Extension<SmithingRecipe>)new AddSocketsExtension());
        final RecipeType<SmithingRecipe> apo_SMITHING = AdventureJEIPlugin.APO_SMITHING;
        final Stream stream = Minecraft.m_91087_().f_91073_.m_7465_().m_44013_(net.minecraft.world.item.crafting.RecipeType.f_44113_).stream();
        final Class<ReactiveSmithingRecipe> obj = ReactiveSmithingRecipe.class;
        Objects.requireNonNull(obj);
        reg.addRecipes((RecipeType)apo_SMITHING, (List)stream.filter(obj::isInstance).toList());
        final List<SalvagingRecipe> salvagingRecipes = new ArrayList<SalvagingRecipe>(Minecraft.m_91087_().f_91073_.m_7465_().m_44013_(Apoth.RecipeTypes.SALVAGING));
        salvagingRecipes.sort(Comparator.comparingInt(recipe -> recipe.getOutputs().stream().mapToInt(SalvagingRecipe.OutputData::getMax).max().orElse(0)));
        reg.addRecipes((RecipeType)AdventureJEIPlugin.SALVAGING, (List)salvagingRecipes);
        final List<GemCuttingCategory.GemCuttingRecipe> gemCutRecipes = new ArrayList<GemCuttingCategory.GemCuttingRecipe>();
        for (final Gem g : GemRegistry.INSTANCE.getValues()) {
            for (LootRarity r = (LootRarity)RarityRegistry.getMinRarity().get(), max = (LootRarity)RarityRegistry.getMaxRarity().get(); r != max; r = r.next()) {
                if (g.clamp(r) == r) {
                    gemCutRecipes.add(new GemCuttingCategory.GemCuttingRecipe(g, r));
                }
            }
        }
        reg.addRecipes((RecipeType)AdventureJEIPlugin.GEM_CUTTING, (List)gemCutRecipes);
    }
    
    public void registerCategories(final IRecipeCategoryRegistration reg) {
        if (!Apotheosis.enableAdventure) {
            return;
        }
        reg.addRecipeCategories(new IRecipeCategory[] { (IRecipeCategory)new ApothSmithingCategory(reg.getJeiHelpers().getGuiHelper()) });
        reg.addRecipeCategories(new IRecipeCategory[] { (IRecipeCategory)new SalvagingCategory(reg.getJeiHelpers().getGuiHelper()) });
        reg.addRecipeCategories(new IRecipeCategory[] { (IRecipeCategory)new GemCuttingCategory(reg.getJeiHelpers().getGuiHelper()) });
    }
    
    public void registerRecipeCatalysts(final IRecipeCatalystRegistration reg) {
        if (!Apotheosis.enableAdventure) {
            return;
        }
        reg.addRecipeCatalyst(new ItemStack((ItemLike)Blocks.f_50625_), new RecipeType[] { AdventureJEIPlugin.APO_SMITHING });
        reg.addRecipeCatalyst(new ItemStack((ItemLike)Adventure.Blocks.SALVAGING_TABLE.get()), new RecipeType[] { AdventureJEIPlugin.SALVAGING });
        reg.addRecipeCatalyst(new ItemStack((ItemLike)Adventure.Blocks.GEM_CUTTING_TABLE.get()), new RecipeType[] { AdventureJEIPlugin.GEM_CUTTING });
    }
    
    public void registerItemSubtypes(final ISubtypeRegistration reg) {
        if (!Apotheosis.enableAdventure) {
            return;
        }
        reg.registerSubtypeInterpreter((Item)Adventure.Items.GEM.get(), (IIngredientSubtypeInterpreter)new GemSubtypes());
    }
    
    static {
        APO_SMITHING = RecipeType.create("apotheosis", "smithing", (Class)AdventureModule.ApothSmithingRecipe.class);
        SALVAGING = RecipeType.create("apotheosis", "salvaging", (Class)SalvagingRecipe.class);
        GEM_CUTTING = RecipeType.create("apotheosis", "gem_cutting", (Class)GemCuttingCategory.GemCuttingRecipe.class);
        DUMMY_INPUTS = Arrays.asList(Items.f_42430_, Items.f_42390_, Items.f_42428_, Items.f_42469_, Items.f_42713_).stream().map((Function<? super Object, ? extends ItemStack>)ItemStack::new).toList();
    }
    
    static class AddSocketsExtension implements ApothSmithingCategory.Extension<AddSocketsRecipe>
    {
        private static final List<ItemStack> DUMMY_OUTPUTS;
        
        @Override
        public void setRecipe(final IRecipeLayoutBuilder builder, final AddSocketsRecipe recipe, final IFocusGroup focuses) {
            builder.addSlot(RecipeIngredientRole.INPUT, 1, 1).addIngredients((IIngredientType)VanillaTypes.ITEM_STACK, (List)AdventureJEIPlugin.DUMMY_INPUTS);
            builder.addSlot(RecipeIngredientRole.INPUT, 50, 1).addIngredients(recipe.getInput());
            builder.addSlot(RecipeIngredientRole.OUTPUT, 108, 1).addItemStacks((List)AddSocketsExtension.DUMMY_OUTPUTS);
        }
        
        @Override
        public void draw(final AddSocketsRecipe recipe, final IRecipeSlotsView recipeSlotsView, final GuiGraphics gfx, final double mouseX, final double mouseY) {
            final Component text = (Component)Component.m_237110_("text.apotheosis.socket_limit", new Object[] { recipe.getMaxSockets() });
            final Font font = Minecraft.m_91087_().f_91062_;
            gfx.m_280614_(font, text, 62 - font.m_92852_((FormattedText)text) / 2, 23, 0, false);
        }
        
        static {
            DUMMY_OUTPUTS = AdventureJEIPlugin.DUMMY_INPUTS.stream().map((Function<? super Object, ?>)ItemStack::m_41777_).map(s -> {
                SocketHelper.setSockets(s, 1);
                return s;
            }).toList();
        }
    }
    
    static class GemSubtypes implements IIngredientSubtypeInterpreter<ItemStack>
    {
        public String apply(final ItemStack stack, final UidContext context) {
            final GemInstance inst = GemInstance.unsocketed(stack);
            if (!inst.isValidUnsocketed()) {
                return ForgeRegistries.ITEMS.getKey((Object)stack.m_41720_()).toString();
            }
            return inst.gem().getId() + "@" + inst.rarity().getId();
        }
    }
}
